<?php

namespace app\abutment\controller;

use app\abutment\validate\Good as GoodValidate;
use app\BaseController;
use app\abutment\logic\Good as GoodLogic;
use think\facade\Db;

//商品模块
class Good extends BaseController
{
    //获取商品列表
    public function list()
    {
        $param = $this->request->filter('trim')->only(['keyword' => '', 'page' => 1, 'size' => 15], 'post');

        validate(GoodValidate::class)->scene('list')->check($param);

        return GoodLogic::list($param);

    }

    //创建商品
    public function createGood()
    {
        $data = [
            'good_name',//商品名称
            'cat_id',//分类
            'brand_id',//品牌
            'good_unit',//单位
            'moq',//起订量
            'customized',//定制工期
            'tax',//税点
            'good_size',//商品尺寸
            'companyNo',//业务公司编码
            'is_auth',//是否有销售权限
            'auth_img' => '',//销售授权图片
            'after_sales',//售后说明
            'craft_desc' => '',//工艺说明
            'good_remark',//商品备注
            'weight',//商品重量
            'packing_way',//包装方式
            'packing_size',//装箱尺寸
            'packing_spec',//装箱规格
            'packing_weight',//装箱重量
            'packing_list',//包装清单,
            'good_bar' => '',//商品条形码
            'supply_area',//供货区域,
            'delivery_place',//发货地,
            'origin_place',//产地',
            'delivery_day',//物流时间,
            'lead_time' => 0,//供货周期
            'cgd_gold_price' => 0,//供应商采购金价
            'sample_day' => 0,//调样周期
            'sample_fee' => 0,//调样费用
            'stock_moq',//库存品备库起订数量
            'is_diff' => '',//是否有工差'
            'good_img',//商品详情主图
            'config' => '',//配置'
            'other_config' => '',//其他配置
            'good_thumb_img',//商品缩略图
            'good_info_img',//商品详情图
            'cert_fee' => 0.00,//证书费
            'packing_fee' => 0.00,//包装费
            'cost_fee' => 0.00,//工艺费
            'mark_fee' => 0.00,//加标费
            'demo_fee' => 0.00,//打样费
            'open_fee' => 0.00,//开模费
            'noble_metal' => 0,//贵金属种类
            'noble_weight' => 0,//贵金属重量
            'is_gold_price' => 0,//是否使用实时金价
            'market_price' => 0,//市场价
            'nake_price' => 0,//含税成本价
            'speclist' => [],//规格列表
            'good_ladder' => [],//阶梯成本列表
            'is_support_barter' => 1,//是否可退货
        ];

        $param = $this->request->filter('trim')->only($data, 'post');
        validate(GoodValidate::class)->scene('create')->check($param);

        $good_name = $this->request->filter('trim')->post('good_name');
        $cat_id = $this->request->filter('trim')->post('cat_id/d');
        $cat = Db::name('cat')->field('id')->where(['id' => $cat_id, 'is_del' => 0])->findOrEmpty();
        if (empty($cat)) return json_show(1004, '分类不存在');
        $brandid = $this->request->filter('trim')->post('brand_id/d');
        $brand = Db::name('brand')->field('id')->where(['id' => $brandid, 'is_del' => 0])->findOrEmpty();
        if (empty($brand)) return json_show(1004, '品牌不存在');
        $unit = $this->request->filter('trim')->post('good_unit/d');
        $unit_info = Db::name('unit')->field('id')->where(['id' => $unit, 'is_del' => 0])->findOrEmpty();
        if (empty($unit_info)) return json_show(1004, '单位不存在');
        $good_type = 2; // 非定制
        $moq = $this->request->filter('trim')->post('moq/d', 0);
        $customized = $this->request->filter('trim')->post('customized/d', 0);
        $is_exclusive = 2;//定制类型，2常规
        $tax = $this->request->filter('trim')->post('tax/d');
        $supplierNo = $this->request->supplier['supplierNo'];
        $good_size = $this->request->filter('trim')->post('good_size');
        $companyNo = $this->request->filter('trim')->post('companyNo');
        $company = Db::name('business')->field('id')->where(['companyNo' => $companyNo, 'is_del' => 0])->findOrEmpty();
        if (empty($company)) return json_show(1004, '业务公司不存在');
        $is_auth = $this->request->filter('trim')->post('is_auth/d');
        $auth_img = $this->request->filter('trim')->post('auth_img', '');

        $after_sales = $this->request->filter('trim')->post('after_sales');
        $craft_desc = $this->request->filter('trim')->post('craft_desc', '');

        $good_remark = $this->request->filter('trim')->post('good_remark');
        $weight = $this->request->filter('trim,floatval')->post('weight');
        $packing_way = $this->request->filter('trim')->post('packing_way');
        $packing_size = $this->request->filter('trim')->post('packing_size');
        $packing_spec = $this->request->filter('trim')->post('packing_spec');
        $packing_weight = $this->request->filter('trim,floatval')->post('packing_weight');
        $packing_list = $this->request->filter('trim')->post('packing_list');
        $good_bar = $this->request->filter('trim')->post('good_bar', '');
        $supply_area = $this->request->filter('trim')->post('supply_area/d');
        $delivery_place = $this->request->filter('trim')->post('delivery_place');//发货地
        $origin_place = $this->request->filter('trim')->post('origin_place');//产地

        $province = Db::name('province')
            ->whereIn('province_code', [$delivery_place[0], $origin_place[0]])
            ->column('id', 'province_code');

        if (empty($province[$delivery_place[0]])) return json_show(1004, '发货地数据不规范，省份编码不存在');
        if (empty($province[$origin_place[0]])) return json_show(1004, '产地数据不规范，省份编码不存在');
        $city = Db::name('city')
            ->where(function ($query) use ($delivery_place) {
                return $query->where(['city_code' => $delivery_place[1], 'province_code' => $delivery_place[0]]);
            })->whereOr(function ($query) use ($origin_place) {
                return $query->where(['city_code' => $origin_place[1], 'province_code' => $origin_place[0]]);
            })
            ->column('id', 'city_code');
        if (empty($city[$delivery_place[1]])) return json_show(1004, '发货地数据不规范，城市编码不存在');
        if (empty($city[$origin_place[1]])) return json_show(1004, '产地数据不规范，城市编码不存在');

        $area = Db::name('area')
            ->where(function ($query) use ($delivery_place) {
                return $query->where(['area_code' => $delivery_place[2], 'city_code' => $delivery_place[1]]);
            })->whereOr(function ($query) use ($origin_place) {
                return $query->where(['area_code' => $origin_place[2], 'city_code' => $origin_place[1]]);
            })
            ->column('id', 'area_code');

        if (empty($area[$delivery_place[2]])) return json_show(1004, '发货地数据不规范，县区编码不存在');
        if (empty($area[$origin_place[2]])) return json_show(1004, '产地数据不规范，县区编码不存在');

        $delivery_day = $this->request->filter('trim')->post('delivery_day/d');
        $lead_time = $this->request->filter('trim')->post('lead_time/d', 0);
        $cgd_gold_price = $this->request->filter('trim,floatval')->post('cgd_gold_price', 0);
        $sample_day = $this->request->filter('trim')->post('sample_day/d', 0);
        $sample_fee = $this->request->filter('trim,floatval')->post('sample_fee', 0);
        $is_stock = 0;//0非库存品
        $stock_moq = $this->request->filter('trim')->post('stock_moq/d', 0);
        $is_diff = $this->request->filter('trim')->post('is_diff/d', 0);
        $good_img = $this->request->filter('trim')->post('good_img');
        $config = $this->request->filter('trim')->post('config', '');
        $other_config = $this->request->filter('trim')->post('other_config', '');
        $good_thumb_img = $this->request->filter('trim')->post('good_thumb_img');
        $good_info_img = $this->request->filter('trim')->post('good_info_img');
        $cert_fee = $this->request->filter('trim,floatval')->post('cert_fee', 0);
        $packing_fee = $this->request->filter('trim,floatval')->post('packing_fee', 0);
        $cost_fee = $this->request->filter('trim,floatval')->post('cost_fee', 0);
        $mark_fee = $this->request->filter('trim,floatval')->post('mark_fee', 0);
        $demo_fee = $this->request->filter('trim,floatval')->post('demo_fee', 0);
        $open_fee = $this->request->filter('trim,floatval')->post('open_fee', 0);
        $noble_weight = $this->request->filter('trim,floatval')->post('noble_weight', 0);
        $market_price = $this->request->filter('trim,floatval')->post('market_price', 0);
        $is_gold_price = $this->request->filter('trim')->post('is_gold_price/d', 0);
        $noble_metal = $this->request->filter('trim')->post('noble_metal/d', 0);
        $nake_price = $this->request->filter('trim,floatval')->post('nake_price', 0);
        $speclist = $this->request->filter('trim')->post('speclist', []);
        $good_ladder = $this->request->filter('trim')->post('good_ladder', []);
        $is_support_barter = $this->request->filter('trim')->post('is_support_barter/d', 1);
        $spucode = makeNo("SKU");
        $data = [
            "spuCode" => $spucode,
            "good_code" => '',
            "good_name" => $good_name,
            "cat_id" => $cat_id,
            'brand_id' => $brandid,
            "good_unit" => $unit,
            "good_type" => $good_type,
            "moq" => $moq,
            "is_exclusive" => $is_exclusive,
            "customized" => $customized,
            "companyNo" => $companyNo,
            "tax" => $tax,
            "supplierNo" => $supplierNo,
            "good_size" => $good_size,
            "is_auth" => $is_auth,
            "is_stock" => $is_stock,
            "auth_img" => $auth_img,
            "after_sales" => $after_sales,
            "craft_desc" => $craft_desc,
            "good_remark" => $good_remark,
            "weight" => $weight,
            "packing_way" => $packing_way,
            "packing_size" => $packing_size,
            "packing_spec" => $packing_spec,
            "packing_list" => $packing_list,
            "packing_weight" => $packing_weight,
            "good_bar" => $good_bar,
            "supply_area" => $supply_area,
            "delivery_place" => implode(',', $delivery_place),
            "origin_place" => implode(',', $origin_place),
            "delivery_day" => $delivery_day,
            "lead_time" => $lead_time,
            "sample_day" => $sample_day,
            "sample_fee" => $sample_fee,
            "good_img" => implode(',', $good_img),
            "good_thumb_img" => $good_thumb_img,
            "good_info_img" => implode(',', $good_info_img),
            "cert_fee" => $cert_fee,
            "packing_fee" => $packing_fee,
            "cost_fee" => $cost_fee,
            "mark_fee" => $mark_fee,
            "demo_fee" => $demo_fee,
            "open_fee" => $open_fee,
            "noble_metal" => $noble_metal,
            "cgd_gold_price" => $cgd_gold_price,
            "noble_weight" => $noble_weight,
            "is_gold_price" => $is_gold_price,
            "market_price" => $market_price,
            "nake_price" => $nake_price,
            "is_diff" => $is_diff,
            "config" => $config,
            "other_config" => $other_config,
            "field_change" => '',
            "is_step" => count($good_ladder) > 1 ? 1 : 0,//如果阶梯价个数大于1，说明启用阶梯价
            "is_online" => 0,
            "stock_moq" => $stock_moq,
            "status" => 1,//1审核通过
            "createrid" => request()->supplier['personid'],
            "creater" => request()->supplier['person'],
            "addtime" => date("Y-m-d H:i:s"),
            "updatetime" => date("Y-m-d H:i:s"),
            'is_support_barter' => $is_support_barter
        ];
        return GoodLogic::createGood($spucode, $data, $speclist, $good_ladder);

    }

    //修改商品基础信息
    public function updateGoodBasicsInfo()
    {
        $param = $this->request->filter('trim')->only([
            'spuCode',//商品spuCode
            'good_name',//商品名称
            'cat_id',//分类id
            'brand_id',//商品品牌id
            'companyNo',//业务公司编码
            'good_unit',//商品单位
            'after_sales',//售后说明
            'craft_desc',//工艺说明
            'good_remark',//商品备注
            'good_size',//商品尺寸
            'weight',//商品重量
            'packing_way',//包装方式
            'packing_size',//装箱尺寸
            'packing_spec',//装箱规格
            'packing_list',//包装清单
            'packing_weight',//装箱重量
            'good_bar',//商品条形码
            'supply_area',//供货区域
            'delivery_place',//发货地
            'origin_place',//产地
            'delivery_day',//物流时间
            'lead_time',//供货周期
            'sample_day',//调样周期
            'good_img',//商品详情主图
            'good_thumb_img',//商品缩略图
            'good_info_img',//商品详情图
            'config',//配置
            'other_config',//其他配置
        ], 'post');

        validate(GoodValidate::class)->scene('UpdateGoodBasicsInfo')->check($param);

        if (isset($param['cat_id'])) {
            $cat = Db::name('cat')->field('id')->where(['id' => $param['cat_id'], 'is_del' => 0])->findOrEmpty();
            if (empty($cat)) return json_show(1004, '分类不存在');
        }

        if (isset($param['brand_id'])) {
            $brand = Db::name('brand')->field('id')->where(['id' => $param['brand_id'], 'is_del' => 0])->findOrEmpty();
            if (empty($brand)) return json_show(1004, '品牌不存在');
        }

        if (isset($param['good_unit'])) {
            $unit_info = Db::name('unit')->field('id')->where(['id' => $param['good_unit'], 'is_del' => 0])->findOrEmpty();
            if (empty($unit_info)) return json_show(1004, '单位不存在');
        }

        if (isset($param['companyNo'])) {
            $company = Db::name('business')->field('id')->where(['companyNo' => $param['companyNo'], 'is_del' => 0])->findOrEmpty();
            if (empty($company)) return json_show(1004, '业务公司不存在');
        }

        if (isset($param['delivery_place']) || isset($param['origin_place'])) {

            $province = Db::name('province')
                ->whereIn('province_code', [$param['delivery_place'][0], $param['origin_place'][0]])
                ->column('id', 'province_code');
            if (empty($province[$param['delivery_place'][0]])) return json_show(1004, '发货地数据不规范，省份编码不存在');
            if (empty($province[$param['origin_place'][0]])) return json_show(1004, '产地数据不规范，省份编码不存在');


            $city = Db::name('city')
                ->where(function ($query) use ($param) {
                    return $query->where(['city_code' => $param['delivery_place'][1], 'province_code' => $param['delivery_place'][0]]);
                })->whereOr(function ($query) use ($param) {
                    return $query->where(['city_code' => $param['origin_place'][1], 'province_code' => $param['origin_place'][0]]);
                })->column('id', 'city_code');
            if (empty($city[$param['delivery_place'][1]])) return json_show(1004, '发货地数据不规范，城市编码不存在');
            if (empty($city[$param['origin_place'][1]])) return json_show(1004, '产地数据不规范，城市编码不存在');

            $area = Db::name('area')
                ->where(function ($query) use ($param) {
                    return $query->where(['area_code' => $param['delivery_place'][2], 'city_code' => $param['delivery_place'][1]]);
                })->whereOr(function ($query) use ($param) {
                    return $query->where(['area_code' => $param['origin_place'][2], 'city_code' => $param['origin_place'][1]]);
                })->column('id', 'area_code');
            if (empty($area[$param['delivery_place'][2]])) return json_show(1004, '发货地数据不规范，县区编码不存在');
            if (empty($area[$param['origin_place'][2]])) return json_show(1004, '产地数据不规范，县区编码不存在');

        }

        return GoodLogic::updateGoodBasicsInfo($param['spuCode'], $param);

    }

    //修改商品价格信息
    public function updateGoodPriceInfo()
    {
        $param = $this->request->filter('trim')->only([
            'spuCode',//商品spuCode
            'tax',//税点
            'sample_fee',//调样费有
            'cert_fee' => 0.00,//证书费
            'packing_fee' => 0.00,//包装费
            'cost_fee' => 0.00,//工艺费
            'mark_fee' => 0.00,//加标费
            'demo_fee' => 0.00,//打样费
            'open_fee' => 0.00,//开模费
            'noble_metal' => 0,//贵金属种类
            'noble_weight' => 0,//贵金属重量
            'is_gold_price' => 0,//是否使用实时金价
            'cgd_gold_price' => 0,//供应商采购金价
            'market_price' => 0,//市场价
            'nake_price' => 0,//含税成本价
//            'speclist' => [],//规格列表
//            'good_ladder' => [],//阶梯成本列表
            'is_support_barter' => 1,//是否可退货
        ], 'post');

        validate(GoodValidate::class)->scene('updateGoodPriceInfo')->check($param);

        //规格,阶梯成本,如何修改？
        return GoodLogic::updateGoodPriceInfo($param['spuCode'], $param);

    }

    //获取商品详情
    public function read()
    {

        $param = $this->request->filter('trim')->only(['spuCode'], 'post');

        validate(GoodValidate::class)->scene('read')->check($param);

        return GoodLogic::read($param['spuCode']);
    }


}