<?php

namespace app\abutment\logic;

use app\admin\model\ActionLog;
use app\admin\model\ProcessOrder;
use think\Exception;
use think\exception\ValidateException;
use think\facade\Db;

//商品上线模块
class GoodUp
{

    //申请商品上线
    public static function create(array $data = [])
    {
        //校验平台是否存在
        $rs = Db::name('platform')
            ->field('id,platform_type')
            ->where(['id' => $data['platform_id'], 'is_del' => 0])
            ->findOrEmpty();
        if (empty($rs)) throw new ValidateException('该平台id不存在');


        //校验是否存在已上线商品
        $res = Db::name('good_platform')
            ->where(['platform_code' => $data['platform_id'], 'is_del' => 0])
            ->where('exam_status', '<=', 6)
            ->whereIn('spuCode', $data['spuCodes'])
            ->column('spuCode');

        if (!empty($res)) return json_show(1005, '以下商品编码已存在该平台的上线申请', $res);

        //校验商品数据
        $good_data = Db::name('good_basic')
            ->field('id,spuCode')
            ->where('is_del', 0)
            ->whereIn('spuCode', $data['spuCodes'])
            ->select()
            ->toArray();

        if (empty($good_data)) return json_show(1005, '未找到商品数据');

        if ($rs['platform_type'] == 1) return \app\youzan\logic\Goodup::create([
            'platform_id' => $data['platform_id'],
            'online_reason' => $data['online_reason'],
            'online_remark' => $data['online_remark'],
            'good_data' => $good_data,
            'createrid' => request()->user['user_id'],
            'creater' => request()->user['nickname'],
        ]);

        $spuCode_count = Db::name("good_nake")
            ->where(["is_del" => 0])
            ->whereIn('spuCode', $data['spuCodes'])
            ->group('spuCode')
            ->column('count(`id`) total', 'spuCode');

        $i = 0;
        Db::startTrans();
        try {
            foreach ($good_data as $value) {

                //改变编码规则，将原来的skuCode后两位换成序列号
                //str_pad字符串填充
                $skuCode = makeNo("SN");
                $skuCode = substr($skuCode, 0, -2) . str_pad($i, 2, '0', STR_PAD_LEFT);
                $tmp = [
                    "spuCode" => $value['spuCode'],
                    "skuCode" => $skuCode,
                    "platform_code" => $data['platform_id'],
                    "online_reason" => $data['online_reason'],
                    "online_remark" => $data['online_remark'],
                    "exam_status" => $spuCode_count[$value['spuCode']] > 0 ? 2 : 1,
                    "is_online" => 0,
                    "status" => 1,
                    "is_del" => 0,
                    "creater" => $value['creater'],
                    "createrid" => $value['createrid'],
                    "addtime" => date("Y-m-d H:i:s"),
                    "updatetime" => date("Y-m-d H:i:s")
                ];
                $i++;
                //修改状态，添加待办
                $create = Db::name("good_platform")->insertGetId($tmp);
                if ($create > 0) {
                    ActionLog::logAdd(['id' => request()->user['user_id'], 'nickname' => request()->user['nickname']], [
                        "order_code" => $skuCode,//咨询单详情编号
                        "status" => $tmp['exam_status'],//这里的status是之前的值
                        "action_remark" => '',//备注
                        "action_type" => "create"//新建create，编辑edit，更改状态status
                    ], "SPSX", $tmp['exam_status'], $tmp);

                    ProcessOrder::AddProcess(['id' => request()->user['user_id'], 'nickname' => request()->user['nickname']], [
                        "order_type" => 'SPSX',
                        "order_code" => $skuCode,//咨询单详情编号
                        "order_id" => $create,
                        "order_status" => $tmp['exam_status'], "before_status" => $tmp['exam_status']
                    ]);
                    $iso = Db::name("good")
                        ->field('id')
                        ->where(["spuCode" => $value['spuCode'], "is_del" => 0])
                        ->find();
                    if ($iso == false) {
                        unset($value['id']);

                        $value['addtime'] = date("Y-m-d H:i:s");
                        $value['updatetime'] = date("Y-m-d H:i:s");
                    } else {
                        $value['id'] = $iso['id'];
                        $value['updatetime'] = date("Y-m-d H:i:s");
                    }
                    $up = Db::name("good")->save($value);
                    if (!$up) throw new Exception('提交失败');
                } else  throw new Exception('提交失败');
            }

            Db::commit();
            return json_show(0, "提交商品上线申请成功");
        } catch (Exception $e) {
            Db::rollback();
            throw new ValidateException('提交商品上线申请失败,' . $e->getMessage());
        }

    }

    //列表
    public static function list(array $param = [])
    {

        $db = Db::name('good_platform')
            ->alias('a')
            ->leftJoin('good_basic b', 'b.spuCode=a.spuCode AND b.is_del=0')
            ->leftJoin('platform p', 'p.id=a.platform_code AND p.is_del=0')
            ->leftJoin('brand c', 'c.id=b.brand_id AND c.is_del=0')
            ->leftJoin('business d', 'd.companyNo=b.companyNo AND d.is_del=0')
            ->where(['a.createrid' => request()->user['user_id'], 'a.is_del' => 0]);

        if ($param['keyword'] != '') $db->whereLike('a.spuCode|b.good_name', '%' . $param['keyword'] . '%');

        $count = $db->count('a.id');

        $list = $db
            ->field('a.id,a.spuCode,a.skuCode,b.good_thumb_img,b.good_name,p.platform_name,a.exam_status,b.cat_id,"" cat_name,b.is_stock,c.brand_name,b.companyNo,d.company,a.plat_code,a.creater,a.addtime')
            ->order(['a.addtime' => 'desc', 'a.id' => 'desc'])
            ->page($param['page'], $param['size'])
            ->withAttr('cat_name', function ($val, $da) {
                return implode('/', array_column(made($da['cat_id']), 'name'));
            })
            ->select()
            ->toArray();

        return json_show(0, '请求成功', ['count' => $count, 'list' => $list]);
    }


}