<?php

namespace app\abutment\middleware;

use think\Exception;
use think\exception\ValidateException;
use think\facade\Cache;
use think\facade\Config;
use think\facade\Db;
use think\facade\Validate;
use think\Response;
use app\abutment\model\RequestLog as RequestLogModel;

//中间件
class apiMiddleware
{

    //白名单
    private $white_list = ['login'];

    //请求入口
    public function handle($request, \Closure $next)
    {

        //请求的唯一标识
        $request->request_id = date('YmdHis') . mt_rand(100000, 999999);

        //1.接收参数
        $param = $request->post();

//        $header = $request->header();

        //判断白名单
        if (!in_array(strtolower(request()->pathinfo()), $this->white_list)) {

//            2.校验相关参数
//            $this->checkParam($param);

//            $this->checkHeader($header);

            //4.获取供应商的开发信息
//            $request->supplier = $this->getSupplierInfoByClientID($header['clientid']);

            //获取用户信息
            $request->user = $this->getUserInfo($param);

//            if (!isset($user['code']) || $user['code'] != 0) throw new ValidateException($user['message']);

//            $request->user = array_merge($user['data'], ['token' => $header['token']]);
            //user_id , nickname
            //5.校验签名
//            $this->checkSign($header, $request->supplier['clientSecret']);
        }

        //3.记录日志
        RequestLogModel::add($request->request_id, [], $param);

        return $next($request);

    }


    //请求结束的回调
    public function end(Response $response)
    {
        //只做记录，不做输出
        RequestLogModel::where('request_id', request()->request_id)->save(['response' => $response->getContent()]);
    }


    //校验相关参数
    private function checkHeader(array $header = [])
    {
        $val = Validate::rule([
            'clientid' => 'require',
            'timestamp|时间戳' => 'require|number|length:10',
            'randomstring|随机字符串' => 'require|alphaNum|length:20',
            'sign|签名' => 'require',
            'token' => 'require|length:40|alphaNum',
        ]);

        if (!$val->check($header)) throw new ValidateException('请求基础参数有误:' . $val->getError());

    }


    //获取供应商的信息
    private function getSupplierInfoByClientID(string $clientID = '')
    {
        $key = Config::get('config.redis_key.supplier_info');

        $key .= $clientID;

        $data = Cache::get($key);

        if (!$data) {

            $data = Db::name('abutment_supplier_development')
                ->alias('a')
                ->field('a.id,a.supplierNo,a.supplier,a.clientID,a.clientSecret,s.person,s.personid')
                ->leftJoin('supplier s', 's.code=a.supplierNo AND s.is_del=0')
                ->where(['a.clientID' => $clientID, 'a.is_del' => 0])
                ->findOrEmpty();

            Cache::set($key, $data, 3600 * 24);
        }

        return $data;
    }


    //校验签名
    private function checkSign(array $header = [], string $clientSecret = '')
    {

        $str = substr($header['randomstring'], 0, 10) . $header['clientid'] . $header['timestamp'] . $header['token'] . substr($header['randomstring'], 10) . $clientSecret;

        //签名 = crypt(随机字符串前10位 . clientID .时间戳 . token . 随机字符串后10位 . $clientSecret, 时间戳);
//        $sign = crypt($str, $header['timestamp']);
        $sign = md5($str);//暂时使用md5，上线以后用crypt

        if ($sign != $header['sign']) throw new ValidateException('签名错误');

    }

    //获取用户信息
    public function getUserInfo(array $param = [])
    {
        $val = Validate::rule([
            'token|用户token' => 'require|length:40|alphaNum',
            'supplierNo|供应商编码' => 'require|length:18|alphaNum',
        ]);
        if (!$val->check($param)) throw new ValidateException('参数错误，' . $val->getError());

        $data = Cache::get(Config::get('redis_key.user_info_token') . $param['token']);
        $data = json_decode($data, true);

        if (!$data) throw new Exception('token已过期');

        $supplierNames = array_column($data['supplier_list'], 'supplierName', 'supplierNo');

        if(!isset($supplierNames[$param['supplierNo']])) throw new Exception('你尚未绑定当前供应商');

        return array_merge($data, ['supplierNo' => $param['supplierNo'], 'supplierName' => $supplierNames[$param['supplierNo']]]);
    }

}