<?php


namespace app\abutment\validate;


use think\Validate;

//商品模块的验证器
class Good extends Validate
{

    //验证规则
    protected $rule = [
        'page|页码' => 'require|number|gt:0',
        'size|每页条数' => 'require|number|gt:0|max:100',
        'keyword|关键字' => 'max:10',
        'good_name|商品名称' => 'require|max:255',
        'spuCode|商品编码' => 'require|alphaNum|length:19',
        'cat_id|分类' => 'require|number|gt:0',
        'brand_id|品牌' => 'require|number|gt:0',
        'good_unit|单位' => 'require|number|gt:0',
        'moq|起订量' => 'number|gt:0',
        'customized|定制工期' => 'number|gt:0|max:10000',
        'tax|税点' => 'require|number|egt:0|max:100',
        'good_size|商品尺寸' => 'require|max:255',
        'companyNo|业务公司编码' => 'require|max:255',
        'is_auth|是否有销售权限' => 'require|number|in:0,1',
        'auth_img|销售授权图片' => 'max:255',
        'after_sales|售后说明' => 'require|max:10000',
        'craft_desc|工艺说明' => 'max:10000',
        'good_remark|商品备注' => 'require|max:10000',
        'weight|商品重量' => 'require|float|max:999999999999.999',
        'packing_way|包装方式' => 'require|max:255',
        'packing_size|装箱尺寸' => 'require|max:255',
        'packing_spec|装箱规格' => 'require|max:255',
        'packing_weight|装箱重量' => 'require|float|max:9999999999999.99',
        'packing_list|包装清单' => 'require|max:255',
        'good_bar|商品条形码' => 'max:255',
        'supply_area|供货区域' => 'require|number|in:1,2',
        'delivery_place|发货地' => 'require|array|length:3',
        'origin_place|产地' => 'require|array|length:3',
        'delivery_day|物流时间' => 'require|number|gt:0|max:10000',
        'lead_time|供货周期' => 'number|gt:0|max:10000',
        'cgd_gold_price|供应商采购金价' => 'float|max:9999999.99',
        'sample_day|调样周期' => 'number|max:9999999',
        'sample_fee|调样费用' => 'float|max:99999999.99',
        'stock_moq|备库起订数量' => 'number|max:9999999',//库存品备库起订量不能为零
        'is_diff|是否有工差' => 'number|in:1,0',
        'good_img|商品详情主图' => 'require|array|max:20',
        'config|配置' => 'max:255',
        'other_config|其他配置' => 'max:255',
        'good_thumb_img|商品缩略图' => 'require|max:255',
        'good_info_img|商品详情图' => 'require|array|max:20',
        'cert_fee|证书费' => 'float|max:99999999.99',
        'packing_fee|包装费' => 'float|max:99999999.99',
        'cost_fee|工艺费' => 'float|max:99999999.99',
        'mark_fee|加标费' => 'float|max:99999999.99',
        'demo_fee|打样费' => 'float|max:99999999.99',
        'open_fee|开模费' => 'float|max:99999999.99',
        'noble_metal|贵金属种类' => 'number|between:0,3',
        'noble_weight|贵金属重量' => 'float|max:9999999.999',
        'is_gold_price|是否使用实时金价' => 'number|in:0,1',
        'market_price|市场价' => 'float|max:99999999.99',
        'nake_price|含税成本价' => 'float|max:99999999.99',
        'speclist|规格列表' => 'array|max:99999999',
        'good_ladder|阶梯成本列表' => 'array|max:99999999',
        'is_support_barter|是否可退货' => 'number|in:0,1',
    ];

    //验证场景
    protected $scene = [
        'list' => ['page', 'size', 'keyword'],//列表
    ];

    //创建商品场景下，移除以下字段
    public function sceneCreate()
    {
        return $this
            ->remove('page', true)
            ->remove('size', true)
            ->remove('id', true)
            ->remove('spuCode', true)
            ->remove('keyword', true);
    }

    //修改商品基础信息场景下，移除以下字段
    public function sceneUpdateGoodBasicsInfo()
    {
        return $this
            ->only(['spuCode', 'good_name', 'cat_id', 'brand_id', 'companyNo', 'good_unit', 'after_sales', 'craft_desc', 'good_remark', 'good_size', 'weight', 'packing_way', 'packing_size', 'packing_spec', 'packing_list', 'packing_weight', 'good_bar', 'supply_area', 'delivery_place', 'origin_place', 'delivery_day', 'lead_time', 'sample_day', 'good_img', 'good_thumb_img', 'good_info_img', 'config', 'other_config'])
            ->remove('good_name', 'require')
            ->remove('cat_id', 'require')
            ->remove('brand_id', 'require')
            ->remove('companyNo', 'require')
            ->remove('good_unit', 'require')
            ->remove('after_sales', 'require')
            ->remove('good_remark', 'require')
            ->remove('good_size', 'require')
            ->remove('weight', 'require')
            ->remove('packing_way', 'require')
            ->remove('packing_size', 'require')
            ->remove('packing_spec', 'require')
            ->remove('packing_list', 'require')
            ->remove('packing_weight', 'require')
            ->remove('supply_area', 'require')
            ->remove('delivery_place', 'require')
            ->remove('origin_place', 'require')
            ->remove('delivery_day', 'require')
            ->remove('good_img', 'require')
            ->remove('good_thumb_img', 'require')
            ->remove('good_info_img', 'require');
    }


    //修改商品价格信息场景下，只校验以下字段
    public function sceneUpdateGoodPriceInfo()
    {
        return $this
            ->only(['spuCode', 'tax', 'sample_fee', 'cert_fee', 'packing_fee', 'cost_fee', 'mark_fee', 'demo_fee', 'open_fee', 'noble_metal', 'noble_weight', 'is_gold_price', 'cgd_gold_price', 'market_price', 'nake_price', 'speclist', 'good_ladder', 'is_support_barter'])
            ->remove('tax', 'require');
    }

    //获取商品详情
    public function sceneRead()
    {
        return $this->only(['spuCode']);
    }


}